/*
    Generate an arch for Radiance

    Paul Bourke
    Auckland University Department of Architecture
*/
#include "stdlib.h"
#include "stdio.h"
#include "math.h"

typedef struct {
    double x,y,z;
} XYZ;

void WriteFacet(XYZ p1, XYZ p2, XYZ p3, XYZ p4);

#define PI       3.141592654
#define TWOPI    6.283185307
#define DTOR     0.01745329252
#define ABS(x)   ((x) < 0 ? -(x) : (x))

char *materialname;
char *objectname;

main(argc,argv)
int argc;
char **argv;
{
    int i,n;
    double r,dy,xstart;
    double theta0,theta1,theta2;
    XYZ p[4];

    if (argc < 7) {
	fprintf(stderr,"Usage: %s ",argv[0]);
	fprintf(stderr,"material name r dy xstart n");
	fprintf(stderr,"\n");
	exit(1);
    }
 
    /* Extract the arguments */
    materialname  = argv[1];
    objectname    = argv[2];
    r = atof(argv[3]);
    dy = atof(argv[4]);
    xstart = atof(argv[5]);
    n = atoi(argv[6]);

    /* Echo the calling sequence */
    printf("# ");
    for (i=0;i<argc;i++)
	printf("%s ",argv[i]);
    printf("\n");

    theta0 = acos(xstart / r);

    /* The top piece */
    p[0].x = xstart; 
    p[1].x = r; 
    p[2].x = r; 
    p[3].x = xstart;

    p[0].y = -dy/2; 
    p[1].y = -dy/2; 
    p[2].y = dy/2; 
    p[3].y = dy/2;

    p[0].z = r * sin(theta0); 
    p[1].z = r * sin(theta0); 
    p[2].z = r * sin(theta0); 
    p[3].z = r * sin(theta0);

    WriteFacet(p[0],p[1],p[2],p[3]);

    /* Write the side piece */
    p[0].x = r; 
    p[1].x = r; 
    p[2].x = r; 
    p[3].x = r;

    p[0].y = -dy/2; 
    p[1].y = -dy/2; 
    p[2].y = dy/2; 
    p[3].y = dy/2;
    
    p[0].z = r * sin(theta0); 
    p[1].z = 0; 
    p[2].z = 0; 
    p[3].z = r * sin(theta0);

    WriteFacet(p[0],p[1],p[2],p[3]);

    /* Create the front and back face */
    for (i=0;i<n;i++) {
	theta1 = theta0 - i * theta0 / (double)n;
	theta2 = theta0 - (i+1) * theta0 / (double)n;

	p[0].x = r * cos(theta1);
	p[1].x = r;
	p[2].x = r;
	p[3].x = r * cos(theta2);

	p[0].y = -dy/2;
	p[1].y = -dy/2;
	p[2].y = -dy/2;
	p[3].y = -dy/2;

	p[0].z = r * sin(theta1);
	p[1].z = p[0].z;
	p[2].z = r * sin(theta2);
	p[3].z = p[2].z;

	WriteFacet(p[0],p[3],p[2],p[1]);

        p[0].y = dy/2;
        p[1].y = dy/2;
        p[2].y = dy/2;
        p[3].y = dy/2;

	WriteFacet(p[0],p[1],p[2],p[3]);
    
	p[1] = p[0];
	p[2] = p[3];

        p[0].y = -dy/2;
        p[3].y = -dy/2;

	WriteFacet(p[0],p[3],p[2],p[1]);
    }

    exit(0);
}

/*
    Write a facet in Radiance format, as a polygon
*/
void WriteFacet(p1,p2,p3,p4)
XYZ p1,p2,p3,p4;
{
    static int num;

    printf("%s polygon %s.%ld\n",materialname,objectname,num++);
    printf("0\n0\n%ld\n",12);
    printf("   %g %g %g\n",p1.x,p1.y,p1.z);
    printf("   %g %g %g\n",p2.x,p2.y,p2.z);
    printf("   %g %g %g\n",p3.x,p3.y,p3.z);
    printf("   %g %g %g\n",p4.x,p4.y,p4.z);
    printf("\n");
}
