/*
    Generate a spiral staircase

    Paul Bourke
    Auckland University Department of Architecture
*/
#include <stdio.h>
#include <math.h>

#define PI       3.141592654
#define TWOPI    6.283185307
#define RTOD     57.29577951
#define DTOR     0.01745329252
#define ABS(x)   ((x) < 0 ? -(x) : (x))

typedef struct {
    double x,y,z;
} XYZ;

char *materialname;
char *objectname;

main(argc,argv)
int argc;
char **argv;
{
    int i,nsteps,railposts,makerails=0;
    double radius,height,thetamax,stepthickness;
    double railthickness,railheight;

    if (argc < 8) {
	fprintf(stderr,"Usage: %s ",argv[0]);
	fprintf(stderr,"mat name ");
	fprintf(stderr,"radius height thetamax nsteps stepthickness\n");
	fprintf(stderr,"   [-r railheight railposts railthickness]\n");
	exit(1);
    }
 
    /* Extract the arguments */
    materialname  = argv[1];
    objectname    = argv[2];
    radius        = atof(argv[3]);
    height        = atof(argv[4]);
    thetamax      = atof(argv[5]);
    nsteps        = atoi(argv[6]);
    stepthickness = atof(argv[7]);

    if (argc > 8) {
	if (argv[8][0] == '-' && argv[8][1] == 'r') {
	    makerails = 1;
	    if (argc >= 12) {
		railheight    = atof(argv[9]);
		railposts     = atoi(argv[10]);
		railthickness = atof(argv[11]);
	    } else {
		fprintf(stderr,"Usage: %s ",argv[0]);
		fprintf(stderr,"mat name ");
		fprintf(stderr,"radius height thetamax nsteps stepthickness\n");
		fprintf(stderr,"   [-r railheight railposts railthickness]\n");
		exit(1);
	    }
	}
    }

    /* Echo the calling sequence */
    printf("# ");
    for (i=0;i<argc;i++)
	printf("%s ",argv[i]);
    printf("\n");

    /* Create the central pole */
    printf("%s cylinder column \n",materialname);
    printf("0\n0\n7 ");
    printf("0 0 0 0 0 %g %g\n",height,radius/10.0);

    /* Create the steps */
    for (i=0;i<nsteps;i++) {
	printf("!genbox %s %s_step.%d ",materialname,objectname,i);
	printf("%g %g %g \\\n",radius,radius/5,stepthickness);
	printf("   | xform -rz %g -t %g %g %g\n",
	    i*thetamax/nsteps,0.0,0.0,i*height/nsteps);
    }

    /* Create the handrails */
    if (makerails == 1) {

	/* Posts */
	for (i=0;i<=railposts;i++) {
	    printf("!genbox %s %s_post.%d ",materialname,objectname,i);
	    printf("%g %g %g \\\n",railthickness,railthickness,railheight);
	    printf("   | xform -t %g %g %g -rz %g\n",
		radius,0.0,i*height/railposts,i*thetamax/railposts);
	}
	
	/* Rail */
	printf("!genworm %s %s_rail ",materialname,objectname);
	printf("\'%g*cos(%g*t*%g)\' ",radius,thetamax,DTOR);
	printf("\'%g*sin(%g*t*%g)\' ",radius,thetamax,DTOR);
	printf("\'%g*t+%g\' ",height,railheight);
	printf("\'%g\' %d\n",railthickness/2,railposts);
    }

    exit(0);
}
